@@ -104,7 +104,6 @@ NNS.caus <- function(x, y,
     }
 
 
-    if(sign(Causation.x.given.y) == sign(Causation.y.given.x)){
       if(abs(Causation.x.given.y) <= abs(Causation.y.given.x)){
         if(plot){
           # For plotting only
@@ -118,7 +117,7 @@ NNS.caus <- function(x, y,
         }
         return(c(Causation.x.given.y = Causation.x.given.y,
                  Causation.y.given.x = Causation.y.given.x,
-                 "C(x--->y)" =  abs(Causation.y.given.x) - abs(Causation.x.given.y)))
+                 "C(x--->y)" = sign(Causation.y.given.x) * (abs(Causation.y.given.x) - abs(Causation.x.given.y))))
       } else {
         if(plot){
           # For plotting only
@@ -132,42 +131,10 @@ NNS.caus <- function(x, y,
         }
         return(c(Causation.x.given.y = Causation.x.given.y,
                  Causation.y.given.x = Causation.y.given.x,
-                 "C(y--->x)" = abs(Causation.x.given.y) - abs(Causation.y.given.x)))
-      }
-    } else {
-      if(Causation.x.given.y <= Causation.y.given.x){
-        if(plot){
-          # For plotting only
-          if(tau == "cs"){
-            tau <- 0
-          }
-          if(tau == "ts"){
-            tau <- 3
-          }
-          Uni.caus(y, x, tau = tau, plot = plot)
-        }
-        return(c(Causation.x.given.y = Causation.x.given.y,
-                 Causation.y.given.x = Causation.y.given.x,
-                 "C(x--->y)" =  Causation.y.given.x - Causation.x.given.y))
-      } else {
-        if(plot){
-          # For plotting only
-          if(tau == "cs"){
-            tau <- 0
-          }
-          if(tau == "ts"){
-            tau <- 3
-          }
-          Uni.caus(x, y, tau = tau, plot = plot)
-        }
-        return(c(Causation.x.given.y = Causation.x.given.y,
-                 Causation.y.given.x = Causation.y.given.x,
-                 "C(y--->x)" = Causation.x.given.y - Causation.y.given.x))
+                 "C(y--->x)" = sign(Causation.x.given.y) * (abs(Causation.x.given.y) - abs(Causation.y.given.x))))
       }
 
 
-    }
-
   } else {
 
     NNS.caus.matrix(x, tau = orig.tau)
