@@ -1,6 +1,7 @@
 #' LPM VaR
 #'
 #' Generates a value at risk (VaR) quantile based on the Lower Partial Moment ratio.
+#'
 #' @param percentile numeric [0, 1]; The percentile for left-tail VaR (vectorized).
 #' @param degree integer; \code{(degree = 0)} for discrete distributions, \code{(degree = 1)} for continuous distributions.
 #' @param x a numeric vector.
@@ -18,6 +19,8 @@
 
 LPM.VaR <- function(percentile, degree, x){
 
+    percentile <- pmax(pmin(percentile, 1), 0)
+
     l <- length(x)
 
     if(degree == 0){
@@ -26,16 +29,16 @@ LPM.VaR <- function(percentile, degree, x){
                       error = quantile(x, percentile))
         return(q)
     } else {
-        sort_x <- sort(x)
-        vars <- LPM.ratio(degree, sort_x, x)
-        index <- findInterval(percentile, vars)
-        vars <- rowMeans(cbind(sort_x[index], sort_x[pmin(l, (index + 1))]))
-        vars[percentile==0] <- min(x)
-        vars[percentile==1] <- max(x)
-        return(vars)
+        func <- function(b){
+            abs(LPM.ratio(degree, b, x) - percentile)
+        }
+        return(optimize(func, c(min(x),max(x)))$minimum)
     }
 }
 
+LPM.VaR <- Vectorize(LPM.VaR, vectorize.args = "percentile")
+
+
 #' UPM VaR
 #'
 #' Generates an upside value at risk (VaR) quantile based on the Upper Partial Moment ratio
@@ -53,6 +56,8 @@ LPM.VaR <- function(percentile, degree, x){
 
 UPM.VaR <- function(percentile, degree, x){
 
+    percentile <- pmax(pmin(percentile, 1), 0)
+
     l <- length(x)
     if(degree==0){
         td <- tdigest::tdigest(x, compression = max(100, log(l,10)*100))
@@ -60,14 +65,13 @@ UPM.VaR <- function(percentile, degree, x){
                       error = quantile(x, 1 - percentile))
         return(q)
     } else {
-        sort_x <- sort(x)
-        vars <- LPM.ratio(degree, sort_x, x)
-        index <- findInterval(1 - percentile, vars)
-        vars <- rowMeans(cbind(sort_x[index], sort_x[pmax(1, (index - 1))]))
-        vars[(1-percentile)==0] <- min(x)
-        vars[(1-percentile)==1] <- max(x)
-        return(vars)
+        func <- function(b){
+            abs(LPM.ratio(degree, b, x) - (1 - percentile))
+        }
+        return(optimize(func, c(min(x),max(x)))$minimum)
     }
 
 }
 
+UPM.VaR <- Vectorize(UPM.VaR, vectorize.args = "percentile")
+
