@@ -3,12 +3,12 @@
 #' Prediction model using the predictions of the NNS base models \link{NNS.reg} as features (i.e. meta-features) for the stacked model.
 #'
 #' @param IVs.train a vector, matrix or data frame of variables of numeric or factor data types.
-#' @param DV.train a numeric or factor vector with compatible dimsensions to \code{(IVs.train)}.
-#' @param IVs.test a vector, matrix or data frame of variables of numeric or factor data types with compatible dimsensions to \code{(IVs.train)}.  If NULL, will use \code{(IVs.train)} as default.
-#' @param type \code{NULL} (default).  To perform a classification of disrete integer classes from factor target variable \code{(DV.train)}, set to \code{(type = "CLASS")}, else for continuous \code{(DV.train)} set to \code{(type = NULL)}.   Like a logistic regression, this setting is not necessary for target variable of two classes e.g. [0, 1].
+#' @param DV.train a numeric or factor vector with compatible dimensions to \code{(IVs.train)}.
+#' @param IVs.test a vector, matrix or data frame of variables of numeric or factor data types with compatible dimensions to \code{(IVs.train)}.  If NULL, will use \code{(IVs.train)} as default.
+#' @param type \code{NULL} (default).  To perform a classification of discrete integer classes from factor target variable \code{(DV.train)}, set to \code{(type = "CLASS")}, else for continuous \code{(DV.train)} set to \code{(type = NULL)}.   Like a logistic regression, this setting is not necessary for target variable of two classes e.g. [0, 1].
 #' @param obj.fn expression; \code{expression(sum((predicted - actual)^2))} (default) Sum of squared errors is the default objective function.  Any \code{expression()} using the specific terms \code{predicted} and \code{actual} can be used.
 #' @param objective options: ("min", "max") \code{"min"} (default) Select whether to minimize or maximize the objective function \code{obj.fn}.
-#' @param dist options:("L1", "L2", "DTW", "FACTOR") the method of distance calculation; Selects the distance calculation used. \code{dist = "L2"} (default) selects the Euclidean distance and \code{(dist = "L1")} seclects the Manhattan distance; \code{(dist = "DTW")} selects the dynamic time warping distance; \code{(dist = "FACTOR")} uses a frequency.
+#' @param dist options:("L1", "L2", "DTW", "FACTOR") the method of distance calculation; Selects the distance calculation used. \code{dist = "L2"} (default) selects the Euclidean distance and \code{(dist = "L1")} selects the Manhattan distance; \code{(dist = "DTW")} selects the dynamic time warping distance; \code{(dist = "FACTOR")} uses a frequency.
 #' @param CV.size numeric [0, 1]; \code{NULL} (default) Sets the cross-validation size if \code{(IVs.test = NULL)}.  Defaults to 0.25 for a 25 percent random sampling of the training set under \code{(CV.size = NULL)}.
 #' @param ts.test integer; NULL (default) Sets the length of the test set for time-series data; typically \code{2*h} parameter value from \link{NNS.ARMA} or double known periods to forecast.
 #' @param folds integer; \code{folds = 5} (default) Select the number of cross-validation folds.
@@ -22,7 +22,7 @@
 #'
 #' @return Returns a vector of fitted values for the dependent variable test set for all models.
 #' \itemize{
-#' \item{\code{"NNS.reg.n.best"}} returns the optimum \code{"n.best"} paramater for the \link{NNS.reg} multivariate regression.  \code{"SSE.reg"} returns the SSE for the \link{NNS.reg} multivariate regression.
+#' \item{\code{"NNS.reg.n.best"}} returns the optimum \code{"n.best"} parameter for the \link{NNS.reg} multivariate regression.  \code{"SSE.reg"} returns the SSE for the \link{NNS.reg} multivariate regression.
 #' \item{\code{"OBJfn.reg"}} returns the \code{obj.fn} for the \link{NNS.reg} regression.
 #' \item{\code{"NNS.dim.red.threshold"}} returns the optimum \code{"threshold"} from the \link{NNS.reg} dimension reduction regression.
 #' \item{\code{"OBJfn.dim.red"}} returns the \code{obj.fn} for the \link{NNS.reg} dimension reduction regression.
@@ -99,9 +99,9 @@ NNS.stack <- function(IVs.train,
     }
   }
 
-  if(dist == "FACTOR"){
-    method <- 1
-  }
+ # if(dist == "FACTOR"){
+ #   method <- 1
+ #  }
 
   objective <- tolower(objective)
 
@@ -114,9 +114,7 @@ NNS.stack <- function(IVs.train,
   if(is.null(IVs.test)){
     IVs.test <- IVs.train
   } else {
-    if(is.null(dim(IVs.test)) || dim(IVs.test)[2]==1){
       IVs.test <- data.frame(IVs.test)
-    }
   }
 
 
@@ -133,11 +131,6 @@ NNS.stack <- function(IVs.train,
   best.nns.cv <- list(folds)
   best.nns.ord <- list(folds)
 
-  if (is.null(ncores)) {
-    num_cores <- detectCores() - 1
-  } else {
-    num_cores <- ncores
-  }
 
   for(b in 1 : folds){
     if(status){
@@ -189,7 +182,7 @@ NNS.stack <- function(IVs.train,
 
       var.cutoffs <- (pmax(var.cutoffs_1, var.cutoffs_2) + pmin(var.cutoffs_1, var.cutoffs_2))/2
 
-      var.cutoffs <- var.cutoffs[var.cutoffs < 1]
+      var.cutoffs <- var.cutoffs[var.cutoffs < 1 & var.cutoffs > 0]
 
       var.cutoffs[is.na(var.cutoffs)] <- 0
 
@@ -235,11 +228,14 @@ NNS.stack <- function(IVs.train,
 
       if(b==folds){
         nns.ord.threshold <- as.numeric(names(sort(table(unlist(THRESHOLDS)), decreasing = TRUE)[1]))
+
         best.nns.ord <- mean(na.omit(unlist(best.nns.ord)))
-        nns.method.2 <- NNS.reg(IVs.train, DV.train,point.est = IVs.test, dim.red.method = dim.red.method, plot = FALSE, order = order, threshold = nns.ord.threshold, ncores = 1,
+
+        nns.method.2 <- NNS.reg(IVs.train, DV.train, point.est = IVs.test, dim.red.method = dim.red.method, plot = FALSE, order = order, threshold = nns.ord.threshold, ncores = 1,
                                 type = type)
 
         rel_vars <- nns.method.2$equation
+
         rel_vars <- rel_vars[rel_vars$Coefficient>0,1][-.N]
 
 
@@ -290,8 +286,8 @@ NNS.stack <- function(IVs.train,
         }
 
         if(index==1){
-          setup <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order, ncores = 1,
-                           type = type, factor.2.dummy = TRUE, dist = dist)
+          setup <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order,
+                           type = type, factor.2.dummy = TRUE, dist = dist, ncores = 1)
           predicted <- setup$Point.est
         } else {
 
@@ -300,21 +296,19 @@ NNS.stack <- function(IVs.train,
 
           if(!is.null(dim(CV.IVs.train))){
             if(dim(CV.IVs.train)[2]>1){
-                cl <- makeCluster(num_cores)
-                registerDoParallel(cl)
 
-                CV.IVs.test <- do.call(cbind, lapply(data.frame(CV.IVs.test), factor_2_dummy_FR))
+                CV.IVs.test.new <- data.table::data.table(do.call(cbind, lapply(data.frame(CV.IVs.test), factor_2_dummy_FR)))
 
-                predicted <- foreach(j = 1:nrow(CV.IVs.test), .packages=c("NNS","data.table","dtw"))%dopar%{
-                    NNS.distance(setup$RPM, dist.estimate = as.vector(CV.IVs.test[j,]), type = dist, k = i)
-                }
+                CV.IVs.test.new <- CV.IVs.test.new[, DISTANCES :=  NNS.distance(setup$RPM, dist.estimate = .SD, type = dist, k = i)[1], by = 1:nrow(CV.IVs.test)]
+
+                predicted <- as.numeric(unlist(CV.IVs.test.new$DISTANCES))
 
-                stopCluster(cl)
-                registerDoSEQ()
             } else {
               predicted <-  NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order, ncores = 1,
                                     type = type, factor.2.dummy = TRUE, dist = dist)$Point.est
             }
+
+            rm(CV.IVs.test.new)
           } else {
             predicted <-  NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order, ncores = 1,
                                   type = type, factor.2.dummy = TRUE, dist = dist)$Point.est
@@ -357,7 +351,7 @@ NNS.stack <- function(IVs.train,
       if(b==folds){
         best.nns.cv <- mean(na.omit(unlist(best.nns.cv)))
         best.k <- round(fivenum(as.numeric(rep(names(table(unlist(best.k))), table(unlist(best.k)))))[4])
-        nns.method.1 <- NNS.reg(IVs.train[ , relevant_vars], DV.train, point.est = IVs.test[, relevant_vars], plot = FALSE, n.best = best.k, order = order, ncores = 1,
+        nns.method.1 <- NNS.reg(IVs.train[ , relevant_vars], DV.train, point.est = IVs.test[, relevant_vars], plot = FALSE, n.best = best.k, order = order, ncores = ncores,
                                 type = type)$Point.est
         if(!is.null(type) && !is.null(nns.method.1)){
           nns.method.1 <- round(nns.method.1)
